//-----------------------------------------------------------------------------
//  (c) 2009 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApi
//  Author:  Fritz Dierks
//  $Header$
//
//  License: This file is published under the license of the EMVA GenICam  Standard Group.
//  A text file describing the legal terms is included in  your installation as 'GenICam_license.pdf'.
//  If for some reason you are missing  this file please contact the EMVA or visit the website
//  (http://www.genicam.org) for a full copy.
//
//  THIS SOFTWARE IS PROVIDED BY THE EMVA GENICAM STANDARD GROUP "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE EMVA GENICAM STANDARD  GROUP
//  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT  LIMITED TO,
//  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  DATA, OR PROFITS;
//  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  THEORY OF LIABILITY,
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)
//  ARISING IN ANY WAY OUT OF THE USE  OF THIS SOFTWARE, EVEN IF ADVISED OF THE
//  POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

/*!
\file
*/


#ifndef __STRUCTPORT_H
#define __STRUCTPORT_H

#include "GenApi/PortImpl.h"

namespace GenApi
{



    // some useful macros for packing structs
#   if defined( _MSC_VER ) && !defined(PACK_STRUCT_STRUCT)
#       define PACK_STRUCT_FIELD(x) x
#       define PACK_STRUCT_STRUCT
#   else  // assume GNU system
#       define PACK_STRUCT_FIELD(x) x __attribute__((packed))
#       define PACK_STRUCT_STRUCT     __attribute__((packed))
#   endif

    //! Implements a register spaces based on a C++ struct
    template <class CDataStruct>
    class CTestPortStruct : public CDataStruct, public GenApi::CPortImpl
    {
    public:
        CTestPortStruct()
        {}

        //-------------------------------------------------------------
        // IBase implementation
        //-------------------------------------------------------------

        //! Get the access mode of the node
        virtual GenApi::EAccessMode GetAccessMode() const
        {
            return RW;
        }

        //! Get the type of the main interface of a node
        virtual GenApi::EInterfaceType GetPrincipalInterfaceType() const
        {
            return intfIPort;
        }

        //---------------------------------------------------------------
        // IPort implementation
        //---------------------------------------------------------------

        //! Reads a chunk of bytes from the port
        virtual void Read(void *pBuffer, int64_t Address, int64_t Length)
        {
            CDataStruct *pDataStruct = static_cast<CDataStruct*>(this);
            if( Address < 0 || Length < 0 || Address + Length > sizeof(CDataStruct) )
                throw RUNTIME_EXCEPTION("CTestPortStruct::Read - Invalid address and/or length");
            memcpy( pBuffer, (uint8_t*)pDataStruct + Address, (size_t)Length );
        }

        //! Writes a chunk of bytes to the port
        virtual void Write(const void *pBuffer, int64_t Address, int64_t Length)
        {
            CDataStruct *pDataStruct = static_cast<CDataStruct*>(this);
            if( Address < 0 || Length < 0 || Address + Length > sizeof(CDataStruct) )
                throw RUNTIME_EXCEPTION("CTestPortStruct::Write - Invalid address and/or length");
            memcpy( (uint8_t*)pDataStruct + Address, pBuffer, (size_t)Length );
        }

    };



};

#endif // __STRUCTPORT_H
